/**********************************************************************
 *
 *       lbox.h  -  Declarations for making little boxes in BGI.
 *
 *  10 Feb 00 Initial write - Bob Larkin
 *  14 May 00 Added annotation to fields - RSL
 *  10 Dec 00 Added check mark - RSL
 *
 **********************************************************************/

#define TRUE  1
#define FALSE 0

#define BACKSPACE	8
#define ENTER		13
#define TAB		9
#define SPACE		32
#define ESC		27
#define CUR_UP		0X4800
#define CUR_DOWN	0X5000
#define CUR_LEFT	0X4B00
#define CUR_RIGHT	0X4D00

/* Bob - Need to declare some global variables with std colors */
#define UBLACK		0
#define UBEIGE		11
#define UWHITE		15

#define BLANK		0		/* Types of lines in boxes */
#define TXT		1
#define BOLD		2		/* Plain now */
#define EDIT		3
#define RADIO		4
#define CHECK		5

#define NON_MODAL	0
#define MODAL		1

#define NON_DISPLAY	0
#define DISPLAY		1

#define ED_INACTIVE	0
#define ED_ACTIVE	1		/* Bit position in edit state */
#define ED_CURSOR_ON	2

#define RADIO_INACTIVE	0		/* Family member not-available */
#define RADIO_PUSH	1
#define RADIO_ACTIVE    2		/* Family member available for push */

#define CHECK_MARK	1		/* Check indicated by bit 0=TRUE */
#define CHECK_ACTIVE    2		/* Check available */

extern char *big_area_ptr1;
extern char *line_ptr1;

/* lb_create(...)  Loads the data structure, but does not draw a box.
 */
extern void lb_create(
	struct lbox_info*,	/* One per box, general info including: */
	char*,			/* Name of box, displayed on top */
	int,			/* Left edge, absolute pixels */
	int,			/* Right edge, absolute pixels */
	int,			/* Top edge, absolute pixels */
	int,			/* Bottom edge, absolute pixels */
	int,			/* First Line - Not Used */
	int);			/* MODAL or NON_MODAL */

/* lb_create_el(...) Loads the data structure for an edit line.
 * The structure edit_info needs to be declared first.
 * Note that the number of characters includes the extra cursor spot at end.
 */
extern void lb_create_el(
	struct edit_info*,	/* One per edit line */
	int,			/* Number of char in field, incl null */
	int,			/* pos - Pixels to right of left box edge */
	char*,			/* Pointer to string to be edited */
	unsigned int,		/* ED_INACTIVE or ED_ACTIVE */
	int,			/* Editable text position in pixels from left*/
	int);			/* Cursor position, in characters 0, 1, 2...*/

/* lb_create_rb(...)  Loads the data structure needed for an entire family
 * of radio buttons. Within a family, checking one button will release the
 * other members of the family.
 * A maximum of 25 buttons per family, for 25 different lines.
 */
extern void lb_create_rb(
	struct radio_info*,	/* One data structure per family */
	int,			/* Button position in pixels from left edge */
	int,			/* Line with pushed button (0, 24) */
	int);			/* Text position in pixels from left edge */

/* lb_create_check(...)  Loads the data structure needed for a check mark.
 */
extern void lb_create_check(
	struct check_info*,	/* One data structure per check mark */
	int,			/* Check position in pixels from left edge */
	unsigned int,		/* State of check, checked=TRUE */
	int);			/* Text position in pixels from left edge */

extern void lb_edit_state(
	struct edit_info*,	/* Which edit field, a pointer */
	unsigned int);		/* ED_INACTIVE or ED_ACTIVE */

extern void lb_radio_active(
	struct radio_info*,	/* Which radio button family, pointer to it */
	int,			/* Line (0, 24) */
	unsigned int);		/* RADIO_INACTIVE, RADIO_ACTIVE or RADIO_PUSH*/

/* Opening a box draws the box outline, but does not fill in the contents.
 * The box must first have been created in lb_create(...).
 */
extern int lb_open(
	struct lbox_info*,	/* A pointer to the box structure */
	int    area);		/* Either 1 (pri) or 2 (msg) for image */

/* The contents of a box are all rewritten by lb_update(...)
 */
extern void lb_update(
	struct lbox_info*);	/* A pointer to the box structure */

/* Each line is defined by lb_line(). This carries the details of the line,
 * but does not actually display it. Must be called before lb_update(...).
 */
extern int lb_line(
	struct lbox_info*,	/* Pointer to the containing box */
	int,			/* The line number (0, 24) */
	int,			/* BLANK, TXT, BOLD, EDIT, RADIO or CHECK */
	char*,			/* Text string for the line */
	void*);			/* Pointer to additional details, defined
				 * for each line type:
				 *   edit_info*   for edit lines
				 *   radio_info*  for radio button families
				 *   check_info*  for check marks
				 *   0            for TEXT or BOLD */

/* lb_edit_line(...) makes the edit line active */
extern unsigned int lb_edit_line(
	struct lbox_info*,	/* Pointer to containing box */
	struct edit_info*);	/* Pointer to edit line structure */

/* lb_radio_button(...) makes the line with a radio button on it
 * active (bold) and the button pushable.
 */
extern unsigned int lb_radio_button(
	struct lbox_info*,	/* The containing box */
	int,			/* The line (0, 24) */
	struct radio_info*);	/* The radio button family structure */

/* lb_check_mark(...) makes the line with a check mark
 * active (bold) and the check changeable. Unlike radio buttons, this
 * is a single line, unrelated to any other.
 */
extern unsigned int lb_check_mark(
	struct lbox_info*,	/* The containing box */
	struct check_info*);	/* The check mark info structure */

/* lb_close(...) undisplays the box and its contents. If the box was non-modal
 * it leaves a hole. If it was modal, the area is redrawn.
 */
extern void lb_close(
	struct lbox_info*,	/* The relevant box */
	int    area);		/* Either 1 (pri) or 2 (msg) for image */

/* lb_msg(...)  -  A modal box for display of messages, such as errors.
 * It has a title in the top bar and 2 lines of message room, with the
 * line "Hit ENTER to continue" at the bottom.
 * Exits, and closes box only on Enter.
 * Box size is fixed, but coordinates are from call.
 * Text lines are 30 characters, maximum.
 */
extern void lb_msg(
	int xleft,		/* Screen coordinates of upper left */
	int ytop,
	char* title,		/* Inserted into top border */
	char* ln1,		/* Two lines of text, up to 30 ch each */
	char* ln2);

/* Returns the character entered to edit field-see lbox.c  */
extern int decision_modalbox(
	int xleft, int ytop,	/* Screen coordinates, upper-left corner */
	char* title,		/* In top border */
	char* ln1, char* ln2, char* ln3, char* ln4);  /* 4 text lines */

/* The dialog box is defined by this structure. The strings are not stored as
 * part of the structure, but pointers to them.
 */
struct lbox_info
	{
	char	name[20];		/* Identifier, printed at top */
	int	modal;			/* TRUE=modal (only activity for
					   interaction) or FALSE for non-modal
					   (information box). */
	int	open;			/* TRUE=open (displayed) */
	int	state;			/* Box state - TBD */
	int	xleft;			/* Box coordinates */
	int	ytop;
	int	xright;
	int	ybottom;
	int	lines;			/* Num text lines-issued when created*/
	int	first_line;		/* Go here when box opened */
	int	line_type[25];		/* 0=Blank
					   1=Text
					   2=Text Bold
					   3=Edit Box
					   4=Radio Button
					   5=Check Mark */
	char	*txt[25];		/* Init to 0; Point to string */
	void	*line_info[25];		/* For edit string, radio, etc */
	};

 /* Note- there are edit_line_nch character positions in the edit box,
    but they are numbered (0, edit_line_nch-1). The last position is
    not available for a real character, since it must hold the cursor
    when the line is full.
  */
 struct edit_info
	{
	int		edit_line_nch;	/* Characters in edit field */
	int		edit_line_pos;	/* Pixel position of left side */
	char*		edit_line;
	unsigned int	state;	/* Bit 0: 0=Dim, 1=Active */
	int     	text_pos;	/* Pixel position of left side */
	int		cursor_pos;	/* Character position, 0 for left ch */
	};

 struct radio_info
	{
	int		button_pos;	/* Pixel position of left side   */
	unsigned int	state[25];	/* Bit 0:  0=Off,      1=Pushed,
					   Bit 1:  0=Inactive, 1=Active
					   Bit 15: 0=Member,   1=Non-mem */
	int     	text_pos;	/* Pixel position of left side   */
	};

 struct check_info
	{
	int		check_pos;	/* Pixel position of left side   */
	unsigned int	state;		/* Bit 0:  0=Off,      1=Checked,
					   Bit 1:  0=Inactive, 1=Active  */
	int     	text_pos;	/* Pixel position of left side   */
	};
